@@ -271,9 +271,9 @@ static const short yyrline[] = { 0,
    366,   373,   378,   385,   392,   396,   403,   414,   418,   423,
    430,   434,   449,   453,   460,   467,   472,   477,   481,   485,
    495,   510,   524,   530,   544,   551,   572,   589,   610,   615,
-   621,   633,   643,   647,   651,   655,   659,   719,   723,   727,
-   734,   739,   744,   749,   754,   758,   763,   769,   777,   781,
-   785,   792,   796,   800,   807,   839,   843,   870
+   621,   633,   643,   647,   651,   655,   659,   717,   721,   725,
+   732,   737,   742,   747,   752,   756,   761,   767,   775,   779,
+   783,   790,   794,   798,   805,   837,   841,   868
 };
 #endif
 
@@ -1491,8 +1491,6 @@ case 57:
 
 		  yyval.i = 0;
 		  if (logged_in) {
-                    syslog(LOG_DEBUG, "his_addr:");
-/*                    dump_sockaddr((struct sockaddr *)&his_addr);  */
 		    memcpy(&data_dest, &his_addr, NRL_SA_LEN((struct sockaddr *)&ctrl_addr));
 
 		    switch(his_addr.sa.sa_family) {
@@ -1542,116 +1540,116 @@ case 57:
 		;
     break;}
 case 58:
-#line 720 "ftpcmd.y"
+#line 718 "ftpcmd.y"
 {
 			yyval.i = FORM_N;
 		;
     break;}
 case 59:
-#line 724 "ftpcmd.y"
+#line 722 "ftpcmd.y"
 {
 			yyval.i = FORM_T;
 		;
     break;}
 case 60:
-#line 728 "ftpcmd.y"
+#line 726 "ftpcmd.y"
 {
 			yyval.i = FORM_C;
 		;
     break;}
 case 61:
-#line 735 "ftpcmd.y"
+#line 733 "ftpcmd.y"
 {
 			cmd_type = TYPE_A;
 			cmd_form = FORM_N;
 		;
     break;}
 case 62:
-#line 740 "ftpcmd.y"
+#line 738 "ftpcmd.y"
 {
 			cmd_type = TYPE_A;
 			cmd_form = yyvsp[0].i;
 		;
     break;}
 case 63:
-#line 745 "ftpcmd.y"
+#line 743 "ftpcmd.y"
 {
 			cmd_type = TYPE_E;
 			cmd_form = FORM_N;
 		;
     break;}
 case 64:
-#line 750 "ftpcmd.y"
+#line 748 "ftpcmd.y"
 {
 			cmd_type = TYPE_E;
 			cmd_form = yyvsp[0].i;
 		;
     break;}
 case 65:
-#line 755 "ftpcmd.y"
+#line 753 "ftpcmd.y"
 {
 			cmd_type = TYPE_I;
 		;
     break;}
 case 66:
-#line 759 "ftpcmd.y"
+#line 757 "ftpcmd.y"
 {
 			cmd_type = TYPE_L;
 			cmd_bytesz = NBBY;
 		;
     break;}
 case 67:
-#line 764 "ftpcmd.y"
+#line 762 "ftpcmd.y"
 {
 			cmd_type = TYPE_L;
 			cmd_bytesz = yyvsp[0].i;
 		;
     break;}
 case 68:
-#line 770 "ftpcmd.y"
+#line 768 "ftpcmd.y"
 {
 			cmd_type = TYPE_L;
 			cmd_bytesz = yyvsp[0].i;
 		;
     break;}
 case 69:
-#line 778 "ftpcmd.y"
+#line 776 "ftpcmd.y"
 {
 			yyval.i = STRU_F;
 		;
     break;}
 case 70:
-#line 782 "ftpcmd.y"
+#line 780 "ftpcmd.y"
 {
 			yyval.i = STRU_R;
 		;
     break;}
 case 71:
-#line 786 "ftpcmd.y"
+#line 784 "ftpcmd.y"
 {
 			yyval.i = STRU_P;
 		;
     break;}
 case 72:
-#line 793 "ftpcmd.y"
+#line 791 "ftpcmd.y"
 {
 			yyval.i = MODE_S;
 		;
     break;}
 case 73:
-#line 797 "ftpcmd.y"
+#line 795 "ftpcmd.y"
 {
 			yyval.i = MODE_B;
 		;
     break;}
 case 74:
-#line 801 "ftpcmd.y"
+#line 799 "ftpcmd.y"
 {
 			yyval.i = MODE_C;
 		;
     break;}
 case 75:
-#line 808 "ftpcmd.y"
+#line 806 "ftpcmd.y"
 {
 			/*
 			 * Problem: this production is used for all pathname
@@ -1682,7 +1680,7 @@ case 75:
 		;
     break;}
 case 77:
-#line 844 "ftpcmd.y"
+#line 842 "ftpcmd.y"
 {
 			int ret, dec, multby, digit;
 
@@ -1707,7 +1705,7 @@ case 77:
 		;
     break;}
 case 78:
-#line 871 "ftpcmd.y"
+#line 869 "ftpcmd.y"
 {
 			if (logged_in)
 				yyval.i = 1;
@@ -1915,7 +1913,7 @@ yyerrhandle:
   yystate = yyn;
   goto yynewstate;
 }
-#line 881 "ftpcmd.y"
+#line 879 "ftpcmd.y"
 
 
 extern jmp_buf errcatch;
