@@ -53,7 +53,7 @@ static char sccsid[] = "@(#)ftpd.c	8.5 (Berkeley) 4/28/95";
 #include <netinet/in.h>
 #include <netinet/in_systm.h>
 #include <netinet/ip.h>
-#ifdef INET6
+#if INET6
 #include <netinet6/in6.h>
 #endif /* INET6 */
 
@@ -98,7 +98,7 @@ static char version[] = "Version 6.00";
 extern	off_t restart_point;
 extern	char cbuf[];
 
-#ifdef INET6
+#if INET6
 #define su union { \
   struct sockaddr sa; \
   struct sockaddr_in sin; \
@@ -118,6 +118,11 @@ struct	sockaddr_in his_addr;
 struct	sockaddr_in pasv_addr;
 #endif /* INET6 */
 
+#ifdef IPSEC
+void *request = NULL;
+int requestlen = 0;
+#endif /* IPSEC */
+
 int	data;
 jmp_buf	errcatch, urgcatch;
 int	logged_in;
@@ -185,7 +190,7 @@ static void	 ack __P((char *));
 static void	 myoob __P((int));
 static int	 checkuser __P((char *));
 static FILE	*dataconn __P((char *, off_t, char *));
-#ifdef INET6
+#if INET6
 static void	 dolog __P((struct sockaddr *));
 #else /* INET6 */
 static void	 dolog __P((struct sockaddr_in *));
@@ -243,17 +248,17 @@ main(argc, argv, envp)
 		exit(1);
 	}
 #ifdef IP_TOS
-#ifdef INET6
+#if INET6
 	if (ctrl_addr.sa.sa_family == AF_INET) {
 #endif /* INET6 */
 	tos = IPTOS_LOWDELAY;
 	if (setsockopt(0, IPPROTO_IP, IP_TOS, (char *)&tos, sizeof(int)) < 0)
 		syslog(LOG_WARNING, "setsockopt (IP_TOS): %m");
-#ifdef INET6
+#if INET6
 	}
 #endif /* INET6 */
 #endif
-#ifdef INET6
+#if INET6
 	memcpy(&data_source, &ctrl_addr, NRL_SA_LEN((struct sockaddr *)&ctrl_addr));
 	switch(ctrl_addr.sa.sa_family) {
 	  case AF_INET:
@@ -288,7 +293,11 @@ main(argc, argv, envp)
 	LastArgv = envp[-1] + strlen(envp[-1]);
 #endif /* SETPROCTITLE */
 
+#ifdef IPSEC
+	while ((ch = getopt(argc, argv, "dlt:T:u:vS:")) != EOF) {
+#else /* IPSEC */
 	while ((ch = getopt(argc, argv, "dlt:T:u:v")) != EOF) {
+#endif /* IPSEC */
 		switch (ch) {
 		case 'd':
 			debug = 1;
@@ -326,6 +335,15 @@ main(argc, argv, envp)
 			debug = 1;
 			break;
 
+#ifdef IPSEC
+		case 'S':
+                        if (nrl_strtoreq(optarg, &request, &requestlen)) {
+                          fprintf(stderr, "ftpd: nrl_strtoreq(%s) failed\n", optarg);
+                          exit(1);
+                        };
+			break;
+#endif /* IPSEC */
+
 		default:
 			warnx("unknown flag -%c ignored", optopt);
 			break;
@@ -347,11 +365,24 @@ main(argc, argv, envp)
 	if (fcntl(fileno(stdin), F_SETOWN, getpid()) == -1)
 		syslog(LOG_ERR, "fcntl F_SETOWN: %m");
 #endif
-#ifdef INET6
+#if INET6
 	dolog((struct sockaddr *)&his_addr);
 #else /* INET6 */
 	dolog(&his_addr);
 #endif /* INET6 */
+#if IPSEC
+        if (!request) {
+          requestlen = 2048; /* XXX */
+          if (!(request = malloc(requestlen))) {
+	    syslog(LOG_ERR, "malloc(%d): %s(%d)", requestlen, strerror(errno), errno);
+            exit(1);
+          };
+          if (getsockopt(0, SOL_SOCKET, SO_SECURITY_REQUEST, request, &requestlen) < 0) {
+	    syslog(LOG_ERR, "getsockopt(..., SO_SECURITY_REQUEST, ...): %s(%d)", strerror(errno), errno);
+	    exit(1);
+          };
+        };
+#endif /* IPSEC */
 	/*
 	 * Set up default state
 	 */
@@ -832,13 +863,19 @@ getdatasock(mode)
 	if (data >= 0)
 		return (fdopen(data, mode));
 	(void) seteuid((uid_t)0);
-#ifdef INET6
+#if INET6
 	s = socket(ctrl_addr.sa.sa_family, SOCK_STREAM, 0);
 #else /* INET6 */
 	s = socket(AF_INET, SOCK_STREAM, 0);
 #endif /* INET6 */
 	if (s < 0)
 		goto bad;
+#ifdef IPSEC
+	if (setsockopt(s, SOL_SOCKET, SO_SECURITY_REQUEST, request, requestlen) < 0) {
+	  syslog(LOG_ERR, "setsockopt(..., SO_SECURITY_REQUEST, ...): %s(%d)\n", strerror(errno), errno);
+          goto bad;
+        };
+#endif /* IPSEC */
 	if (setsockopt(s, SOL_SOCKET, SO_REUSEADDR,
 	    (char *) &on, sizeof(on)) < 0)
 		goto bad;
@@ -849,7 +886,7 @@ getdatasock(mode)
 #endif /* INET6 */
 	for (tries = 1; ; tries++) {
 		if (bind(s, (struct sockaddr *)&data_source,
-#ifdef INET6
+#if INET6
 		    NRL_SA_LEN((struct sockaddr *)&data_source)) >= 0)
 #else /* INET6 */
 		    sizeof(data_source)) >= 0)
@@ -861,13 +898,13 @@ getdatasock(mode)
 	}
 	(void) seteuid((uid_t)pw->pw_uid);
 #ifdef IP_TOS
-#ifdef INET6
+#if INET6
 	if (data_source.sa.sa_family == AF_INET) {
 #endif /* INET6 */
 	on = IPTOS_THROUGHPUT;
 	if (setsockopt(s, IPPROTO_IP, IP_TOS, (char *)&on, sizeof(int)) < 0)
 		syslog(LOG_WARNING, "setsockopt (IP_TOS): %m");
-#ifdef INET6
+#if INET6
 	}
 #endif /* INET6 */
 #endif
@@ -898,7 +935,7 @@ dataconn(name, size, mode)
 	else
 		(void) strcpy(sizebuf, "");
 	if (pdata >= 0) {
-#ifdef INET6
+#if INET6
 	  su from;
 #else /* INET6 */
 		struct sockaddr_in from;
@@ -915,13 +952,13 @@ dataconn(name, size, mode)
 		(void) close(pdata);
 		pdata = s;
 #ifdef IP_TOS
-#ifdef INET6
+#if INET6
 		if (from.sa.sa_family == AF_INET) {
 #endif /* INET6 */
 		tos = IPTOS_LOWDELAY;
 		(void) setsockopt(s, IPPROTO_IP, IP_TOS, (char *)&tos,
 		    sizeof(int));
-#ifdef INET6
+#if INET6
 		}
 #endif /* INET6 */
 #endif
@@ -936,7 +973,7 @@ dataconn(name, size, mode)
 		return (fdopen(data, mode));
 	}
 	if (usedefault)
-#ifdef INET6
+#if INET6
 	  memcpy(&data_dest, &his_addr, NRL_SA_LEN((struct sockaddr *)&his_addr));
 #else /* INET6 */
 		data_dest = his_addr;
@@ -944,7 +981,7 @@ dataconn(name, size, mode)
 	usedefault = 1;
 	file = getdatasock(mode);
 	if (file == NULL) {
-#ifdef INET6
+#if INET6
 	  char addr[46], port[6];
 	  if (nrl_getnameinfo((struct sockaddr *)&data_source, NRL_SA_LEN((struct sockaddr *)&data_source), addr, sizeof(addr), port, sizeof(port), 3))
 	    reply(425, "Can't create data socket: %s.");
@@ -959,7 +996,7 @@ dataconn(name, size, mode)
 	}
 	data = fileno(file);
 	while (connect(data, (struct sockaddr *)&data_dest,
-#ifdef INET6
+#if INET6
 	    NRL_SA_LEN((struct sockaddr *)&data_dest)) < 0) {
 #else /* INET6 */
 	    sizeof(data_dest)) < 0) {
@@ -1176,7 +1213,7 @@ statfilecmd(filename)
 void
 statcmd()
 {
-#ifdef INET6
+#if INET6
   struct sockaddr *sa;
 #else /* INET6 */
 	struct sockaddr_in *sin;
@@ -1186,7 +1223,7 @@ statcmd()
 	lreply(211, "%s FTP server status:", hostname, version);
 	printf("     %s\r\n", version);
 	printf("     Connected to %s", remotehost);
-#ifdef INET6
+#if INET6
         {
 	  char buf[46];
 	  if (!nrl_getnameinfo((struct sockaddr *)&his_addr, NRL_SA_LEN((struct sockaddr *)&his_addr), buf, sizeof(buf), NULL, 0, 1))
@@ -1224,7 +1261,7 @@ statcmd()
 		printf("     Data connection open\r\n");
 	else if (pdata != -1) {
 		printf("     in Passive mode");
-#ifdef INET6
+#if INET6
 		sa = (struct sockaddr *)&pasv_addr;
 #else /* INET6 */
 		sin = &pasv_addr;
@@ -1232,13 +1269,13 @@ statcmd()
 		goto printaddr;
 	} else if (usedefault == 0) {
 		printf("     PORT");
-#ifdef INET6
+#if INET6
 		sa = (struct sockaddr *)&data_dest;
 #else /* INET6 */
 		sin = &data_dest;
 #endif /* INET6 */
 printaddr:
-#ifdef INET6
+#if INET6
 		{
 		  char buf[128];
 		  if (!satosport(buf, sa))
@@ -1448,7 +1485,7 @@ renamecmd(from, to)
 }
 
 static void
-#ifdef INET6
+#if INET6
 dolog(sa)
   struct sockaddr *sa;
 #else /* INET6 */
@@ -1456,7 +1493,7 @@ dolog(sin)
 	struct sockaddr_in *sin;
 #endif /* INET6 */
 {
-#ifdef INET6
+#if INET6
   if (nrl_getnameinfo(sa, NRL_SA_LEN(sa), remotehost, sizeof(remotehost), NULL, 0, 0)) {
     syslog(LOG_ERR, "getnameinfo failed, bailing");
     exit(1);
@@ -1534,7 +1571,7 @@ myoob(signo)
  *	with Rick Adams on 25 Jan 89.
  */
 void
-#ifdef INET6
+#if INET6
 passive(l)
   int l;
 #else /* INET6 */
@@ -1544,7 +1581,7 @@ passive()
 	int len;
 	char *p, *a;
 
-#ifdef INET6
+#if INET6
 	pdata = socket(ctrl_addr.sa.sa_family, SOCK_STREAM, 0);
 #else /* INET6 */
 	pdata = socket(AF_INET, SOCK_STREAM, 0);
@@ -1553,7 +1590,13 @@ passive()
 		perror_reply(425, "Can't open passive connection");
 		return;
 	}
-#ifdef INET6
+#ifdef IPSEC
+	if (setsockopt(pdata, SOL_SOCKET, SO_SECURITY_REQUEST, request, requestlen) < 0) {
+	  syslog(LOG_ERR, "setsockopt(..., SO_SECURITY_REQUEST, ...): %s(%d)\n", strerror(errno), errno);
+          goto pasv_error;
+        };
+#endif /* IPSEC */
+#if INET6
 	memcpy(&pasv_addr, &ctrl_addr, NRL_SA_LEN((struct sockaddr *)&ctrl_addr));
 	switch(ctrl_addr.sa.sa_family) {
 	  case AF_INET:
@@ -1570,7 +1613,7 @@ passive()
 	pasv_addr.sin_port = 0;
 #endif /* INET6 */
 	(void) seteuid((uid_t)0);
-#ifdef INET6
+#if INET6
 	if (bind(pdata, (struct sockaddr *)&pasv_addr, NRL_SA_LEN((struct sockaddr *)&pasv_addr)) < 0) {
 #else /* INET6 */
 	if (bind(pdata, (struct sockaddr *)&pasv_addr, sizeof(pasv_addr)) < 0) {
@@ -1584,7 +1627,7 @@ passive()
 		goto pasv_error;
 	if (listen(pdata, 1) < 0)
 		goto pasv_error;
-#ifdef INET6
+#if INET6
 	{
 	  char buffer[80];
 
