@@ -66,6 +66,7 @@ Research Laboratory (NRL).
 
 #include <stdio.h>
 #include <sys/types.h>
+#include <sys/param.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netinet/tcp.h>
@@ -77,6 +78,13 @@ Research Laboratory (NRL).
 #include <string.h>
 #include <stdarg.h>
 #include <stdlib.h>
+#include <sys/uio.h>
+
+#if INET6
+#include <netinet6/in6.h>
+#endif /* INET6 */
+
+#include <sys/un.h>
 
 #if FASTCTO
 #include <signal.h>
@@ -94,7 +102,6 @@ struct nrl_nametonum socktypes[] = {
   { 0, NULL, 0 }
 };
 
-int port = 7777;
 char *portname = NULL;
 char *hostname = NULL;
 
@@ -105,19 +112,35 @@ int patternlen = 0;
 
 int nflag = 0;
 
-char addrbuf[128], addrbuf2[128];
-struct sockaddr *addr;
+typedef union {
+  struct sockaddr sa;
+  struct sockaddr_in sin;
+  struct sockaddr_un sun;
+#if INET6
+  struct sockaddr_in6 sin6;
+#endif /* INET6 */
+} su;
+char addrbuf[sizeof(su)], addrbuf2[sizeof(su)];
 int addrlen;
 
+#if INET6 && defined(IPV6_PKTINFO)
+struct in6_pktinfo in6_pktinfo;
+#endif /* INET6 && defined(IPV6_PKTINFO) */
+
+char controlbuf[1024];
+
 char *datablock;
 int datablocklen = 65535;
 char mynamestr[256];
+char messagebuf[1024];
+
+char *myname;
 
 char *requeststr = NULL;
 
-#ifdef ADDRFORM
+#if INET6 && defined(IPV6_ADDRFORM)
 int aform = 0;
-#endif /* ADDRFORM */
+#endif /* INET6 && defined(IPV6_ADDRFORM) */
 
 #ifdef FASTCTO
 static jmp_buf timeout_env;
@@ -140,9 +163,8 @@ void say(char *message, ...)
   putchar('\n');
 }
 
-#ifdef ADDRFORM
-addrform(form)
-int form;
+#ifdef IPV6_ADDRFORM
+void addrform(int form)
 {
   int len, old, new;
 
@@ -169,12 +191,13 @@ int form;
     perror("socktest: getsockname");
     exit(1);
   }
-  addr = (struct sockaddr *)addrbuf;
 
   if (new != form) {
     fprintf(stderr, "socktest: address form error: started with %s, requested %s\n", nrl_afnumtoname(old), nrl_afnumtoname(form));
     fprintf(stderr, "socktest: address form error: but ended up with %s!\n", nrl_afnumtoname(new));
   }
+
+  af = form;
 }
 #endif /* ADDRFORM */
 
@@ -184,6 +207,9 @@ usage(char *myname)
 #ifdef ADDRFORM
   fprintf(stderr, " [-f address_form]");
 #endif /* ADDRFORM */
+#if IPSEC
+  fprintf(stderr, " [-S security_request]");
+#endif /* IPSEC */
   fprintf(stderr," [-s] [host] [port]\n");
   exit(1);
 }
@@ -194,11 +220,18 @@ main(argc, argv)
 {
   int i;
   char *p;
-  struct addrinfo *ai, *ai2;
+  struct addrinfo req, *ai, *ai2;
   char ch;
   int ahlev = 0, esplev = 0, esptlev = 0;
-  char hbuf[32], sbuf[8];
+  char hbuf[32], sbuf[8], dbuf[32], ibuf[16];
+  struct msghdr msghdr, msghdr2;
+  struct iovec iovec;
  
+  if (myname = strrchr(argv[0], '/'))
+    *(myname++) = 0;
+  else
+    myname = argv[0];
+
   while ((ch = getopt(argc, argv, "S:A:N:T:a:f:t:snp:b:")) != EOF)
     switch(ch) {
     case 'a':
@@ -215,12 +248,17 @@ main(argc, argv)
       }
       break;
 #endif /* ADDRFORM */
+#if IPSEC
+    case 'S':
+      requeststr = optarg;
+      break;
+#endif /* IPSEC */
     case 's':
       server = 1;
       break;
     case 't':
       if ((type = nrl_nametonum(socktypes, optarg)) == -1) {
-	fprintf(stderr,"socktest: invalid socket type: %s\n", optarg);
+	fprintf(stderr, "%s: invalid socket type: %s\n", myname, optarg);
 	exit(1);
       }
       break;
@@ -229,18 +267,18 @@ main(argc, argv)
       break;
     case 'b':
       if (!(datablocklen = atoi(optarg))) {
-        fprintf(stderr, "socktest: %s: invalid buffer size\n", optarg);
+        fprintf(stderr, "%s: %s: invalid buffer size\n", myname, optarg);
         exit(1);
       }
       break;
     case 'p':
       if (!(patternlen = atoi(optarg))) {
-        fprintf(stderr, "socktest: %s: invalid test pattern start\n", optarg);
+        fprintf(stderr, "%s: %s: invalid test pattern start\n", myname, optarg);
         exit(1);
       }
       break;
     default:
-      usage(argv[0]);
+      usage(myname);
     }
 
   if (optind < argc)
@@ -255,8 +293,25 @@ main(argc, argv)
   if (!portname)
     portname = "7777";
 
-  {
-  struct addrinfo req;
+#if INET6 && defined(IPV6_PKTINFO)
+  if (optind < argc)
+    if (!(in6_pktinfo.ipi6_ifindex = if_nametoindex(argv[optind]))) {
+      fprintf(stderr, "%s: %s: unknown/invalid interface name\n", myname, argv[optind]);
+      exit(1);
+    };
+
+  optind++;
+
+  if (optind < argc) {
+    memset(&req, 0, sizeof(struct addrinfo));
+    req.ai_family = AF_INET6;
+    if (i = getaddrinfo(argv[optind], NULL, &req, &ai)) {
+      fprintf(stderr, "%s: getaddrinfo: %s.%s: %s\n", myname, argv[optind], NULL, nrl_gai_strerror(i));
+      exit(1);
+    };
+    in6_pktinfo.ipi6_addr = ((struct sockaddr_in6 *)(ai->ai_addr))->sin6_addr;
+  };
+#endif /* INET6 && defined(IPV6_PKTINFO) */
 
   memset(&req, 0, sizeof(struct addrinfo));
 
@@ -266,72 +321,115 @@ main(argc, argv)
     req.ai_flags |= AI_PASSIVE;
 
   if (i = getaddrinfo(hostname, portname, &req, &ai)) {
-    fprintf(stderr, "socktest: getaddrinfo: %s.%s: %s\n", hostname, portname, nrl_gai_strerror(i));
+    fprintf(stderr, "%s: getaddrinfo: %s.%s: %s\n", myname, hostname, portname, nrl_gai_strerror(i));
     exit(1);
   }
-  }
 
 #ifdef FASTCTO
   signal(SIGALRM, &timeout_handler);
 #endif /* FASTCTO */
 
-  {
-    for (ai2 = ai, s = -1; ai; ai = ai->ai_next) {
-      if (nrl_getnameinfo(ai->ai_addr, ai->ai_addrlen, hbuf, sizeof(hbuf), sbuf, sizeof(sbuf), 0)) {
-	printf("socktest: getnameinfo() failed!\n");
+  for (ai2 = ai, s = -1; ai; ai = ai->ai_next) {
+    if (nrl_getnameinfo(ai->ai_addr, ai->ai_addrlen, hbuf, sizeof(hbuf), sbuf, sizeof(sbuf), 0)) {
+      printf("%s: getnameinfo() failed!\n", myname);
+      continue;
+    }
+
+    printf("Trying %s.%s...\n", hbuf, sbuf);
+    
+    if ((s = socket(af = ai->ai_family, ai->ai_socktype, ai->ai_protocol)) < 0) {
+      perror("socket");
+      continue;
+    }
+
+#if IPSEC
+    if (requeststr) {
+      void *request, *request2;
+      int requestlen, requestlen2;
+
+      if (nrl_strtoreq(requeststr, &request, &requestlen)) {
+	fprintf(stderr, "nrl_strtoreq() failed!\n");
+	exit(1);
+      };
+      
+      if (setsockopt(s, SOL_SOCKET, SO_SECURITY_REQUEST, request, requestlen) < 0) {
+	fprintf(stderr, "%s: setsockopt(..., SO_SECURITY_REQUEST, ...): %s(%d)", myname, strerror(errno), errno);
+	exit(1);
+      };
+
+      if (!(request2 = malloc(requestlen2 = requestlen))) {
+	fprintf(stderr, "%s: malloc(%d): %s(%d)", myname, requestlen, strerror(errno), errno);
+	exit(1);
+      };
+
+      if (getsockopt(s, SOL_SOCKET, SO_SECURITY_REQUEST, request2, &requestlen2) < 0) {
+	fprintf(stderr, "%s: getsockopt(..., SO_SECURITY_REQUEST, ...): %s(%d)", myname, strerror(errno), errno);
+	exit(1);
+      };
+
+      if ((requestlen != requestlen2) || memcmp(request, request2, requestlen)) {
+        fprintf(stderr, "%s: kernel changed request!\n", myname);
+        exit(1);
+      };
+
+      {
+        char buffer[8];
+
+        if (nrl_reqtostr(request, requestlen, buffer, sizeof(buffer))) {
+          fprintf(stderr, "%s: nrl_reqtostr failed\n", myname);
+          exit(1);
+        };
+
+        printf("%s: made security request of '%s'\n", myname, buffer);
+      };
+
+      free(request2);
+      free(request);
+    };
+#endif /* IPSEC */
+
+    if (ai->ai_flags & AI_PASSIVE) {
+      if (bind(s, ai->ai_addr, ai->ai_addrlen) < 0) {
+	perror("bind");
+	close(s);
+	s = -1;
 	continue;
       }
 
-      printf("Trying %s.%s...\n", hbuf, sbuf);
-
-      if ((s = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol)) < 0) {
-	perror("socket");
+      if ((listen(s, 1) < 0) && (errno != EOPNOTSUPP)) {
+	perror("listen");
+	close(s);
+	s = -1;
 	continue;
       }
-
-      if (ai->ai_flags & AI_PASSIVE) {
-	if (bind(s, ai->ai_addr, ai->ai_addrlen) < 0) {
-	  perror("bind");
-	  close(s);
-	  s = -1;
-	  continue;
-	}
-
-	if ((listen(s, 1) < 0) && (errno != EOPNOTSUPP)) {
-	  perror("listen");
-	  close(s);
-	  s = -1;
-	  continue;
-	}
-      } else {
+    } else {
 #if FASTCTO
-	if (setjmp(timeout_env)) {
-	  fprintf(stderr, "socktest: Connection timed out\n");
-	  continue;
-	}
-	
-	alarm(FASTCTO);
+      if (setjmp(timeout_env)) {
+	fprintf(stderr, "%s: Connection timed out\n", myname);
+	continue;
+      }
+      
+      alarm(FASTCTO);
 #endif /* FASTCTO */
-
-	if (connect(s, ai->ai_addr, ai->ai_addrlen) < 0) {
-#ifdef FASTCTO
-	  alarm(0);
-#endif /* FASTCTO */
-	  perror("connect");
-	  close(s);
-	  s = -1;
-	  continue;
-	}
+      
+      if (connect(s, ai->ai_addr, ai->ai_addrlen) < 0) {
 #ifdef FASTCTO
 	alarm(0);
 #endif /* FASTCTO */
+	perror("connect");
+	close(s);
+	s = -1;
+	continue;
       }
-      break;
+#ifdef FASTCTO
+      alarm(0);
+#endif /* FASTCTO */
     }
-    if (s < 0)
-      exit(1);
+    break;
   }
-  
+  if (s < 0)
+    exit(1);
+
   if (ai->ai_protocol == IPPROTO_TCP) {
     int val = 1;
     if (setsockopt(s, IPPROTO_TCP, TCP_NODELAY, &val, sizeof(val)) < 0) {
@@ -340,11 +438,22 @@ main(argc, argv)
     }
   }
 
+  af = ai->ai_family;
 
-#ifdef ADDRFORM
+#if INET6 && defined(IPV6_ADDRFORM)
   if (aform && (aform != af))
-	addrform(aform);
-#endif /* ADDRFORM */
+    addrform(aform);
+#endif /* INET6 && defined(IPV6_ADDRFORM) */
+
+#if INET6 && defined(IPV6_PKTINFO)
+  if (af == AF_INET6) {
+    int val = 1;
+    if (setsockopt(s, IPPROTO_IPV6, IPV6_PKTINFO, &val, sizeof(val)) < 0) {
+      perror("setsockopt (IPV6_PKTINFO)");
+      exit(1);
+    }
+  };
+#endif /* INET6 && defined(IPV6_PKTINFO) */
 
   if (nrl_getnameinfo(ai->ai_addr, ai->ai_addrlen, hbuf, sizeof(hbuf), sbuf, sizeof(sbuf), nflag)) {
     fprintf(stderr, "getnameinfo() failed!\n");
@@ -356,20 +465,42 @@ main(argc, argv)
   sprintf(mynamestr, "%s [%s.%s]:\n\t", server ? "Server" : "Client", hbuf, sbuf);
 
   if (patternlen > datablocklen) {
-    fprintf(stderr, "socktest: invalid pattern length\n");
+    fprintf(stderr, "%s: invalid pattern length\n", myname);
     exit(1);
   };
 
   if (!(datablock = malloc(datablocklen + 1))) {
-    fprintf(stderr, "socktest: can't allocate %d bytes buffer space\n", datablocklen);
+    fprintf(stderr, "%s: can't allocate %d bytes buffer space\n", myname, datablocklen);
     exit(1);
   };
 
+  memset(&msghdr, 0, sizeof(struct msghdr));
+  msghdr.msg_iov = &iovec;
+  msghdr.msg_iovlen = 1;
+  msghdr.msg_flags = 0;
+
+  iovec.iov_base = datablock;
+  iovec.iov_len = datablocklen - 1;
+
   say("Ready");
 
   if (server)
     goto servercode;
 
+  if (ai->ai_family == AF_INET6) {
+    msghdr.msg_control = controlbuf;
+    msghdr.msg_controllen = sizeof(struct cmsghdr) + sizeof(struct in6_pktinfo);
+
+    ((struct cmsghdr *)controlbuf)->cmsg_len = sizeof(struct cmsghdr) + sizeof(struct in6_pktinfo);
+    ((struct cmsghdr *)controlbuf)->cmsg_level = IPPROTO_IPV6;
+    ((struct cmsghdr *)controlbuf)->cmsg_type = IPV6_PKTINFO;
+
+    memcpy(controlbuf + sizeof(struct cmsghdr), &in6_pktinfo, sizeof(struct in6_pktinfo));
+  } else {
+    msghdr.msg_control = NULL;
+    msghdr.msg_controllen = 0;
+  };
+
   if (patternlen)
     goto patterncode;
 
@@ -379,10 +510,9 @@ main(argc, argv)
     if (p = strchr(datablock, '\n'))
       *p = 0;
 
-    datasize = strlen(datablock);
-
-    if ((write(s, datablock, datasize)) < 0) { 
-      perror("write");
+    iovec.iov_len = strlen(datablock);
+    if (sendmsg(s, &msghdr, 0) < 0) {
+      perror("sendmsg"); 
       exit(1);
     };
 
@@ -390,8 +520,11 @@ main(argc, argv)
 
     fflush(stdout);
 
-    if ((datasize = read(s, datablock, datablocklen)) < 0) {
-      perror("read");
+    memcpy(&msghdr2, &msghdr, sizeof(struct msghdr));
+    msghdr2.msg_controllen = sizeof(controlbuf);
+
+    if ((datasize = recvmsg(s, &msghdr2, 0)) < 0) {
+      perror("recvmsg");
       exit(1);
     };
 
@@ -424,13 +557,13 @@ patterncode:
     i = 0;
 
     if (datasize != patternlen) {
-      fprintf(stderr, "socktest: Length mismatch (%d != %d)\n", patternlen, datasize);
+      fprintf(stderr, "%s: Length mismatch (%d != %d)\n", myname, patternlen, datasize);
       i = patternlen;
     }
 
     while(i < patternlen) {
       if (datablock[i] != '0' + (i % 10)) {
-        fprintf(stderr, "socktest: Data mismatch at byte %d\n", i);
+        fprintf(stderr, "%s: Data mismatch at byte %d\n", myname, i);
         i = patternlen;
       }
       i++;
@@ -442,6 +575,11 @@ patterncode:
 
 servercode:
   s2 = -1;
+  dbuf[0] = ibuf[0] = 0;
+  msghdr.msg_name = addrbuf;
+  msghdr.msg_namelen = sizeof(addrbuf);
+  msghdr.msg_control = controlbuf;
+  msghdr.msg_controllen = sizeof(controlbuf);
 
   while(1) {
     if (!datasize) {
@@ -470,33 +608,73 @@ servercode:
       }
     }
 
-    addrlen = sizeof(addrbuf);
-    if ((datasize = recvfrom(s2, datablock, datablocklen - 1, 0,
-			     (struct sockaddr *)addrbuf, &addrlen)) < 0) {
-      perror("recvfrom");
+    msghdr2 = msghdr;
+    iovec.iov_len = datablocklen - 1;
+
+    if ((datasize = recvmsg(s2, &msghdr2, 0)) < 0) {
+      perror("recvmsg");
       exit(1);
     }
 
     if (datasize && memcmp(addrbuf, addrbuf2, sizeof(addrbuf))) {
       memcpy(addrbuf2, addrbuf, sizeof(addrbuf));
 
-      if (nrl_getnameinfo((struct sockaddr *)addrbuf, addrlen, hbuf, sizeof(hbuf), sbuf, sizeof(sbuf), nflag)) {
+      if (nrl_getnameinfo((struct sockaddr *)addrbuf, msghdr2.msg_namelen, hbuf, sizeof(hbuf), sbuf, sizeof(sbuf), nflag)) {
 	fprintf(stderr, "getnameinfo() failed!\n");
 	exit(1);
-      }
-    }
+      };
+    };
+
+#if INET6 && defined(IPV6_PKTINFO)
+    {
+      struct cmsghdr *cmsghdr;
+      for (cmsghdr = CMSG_FIRSTHDR(&msghdr2); cmsghdr; cmsghdr = CMSG_NXTHDR(&msghdr2, cmsghdr))
+	if ((cmsghdr->cmsg_level == IPPROTO_IPV6) && (cmsghdr->cmsg_type == IPV6_PKTINFO))
+	  if (memcmp(CMSG_DATA(cmsghdr), &in6_pktinfo, sizeof(struct in6_pktinfo))) {
+	    struct sockaddr_in6 sin6;
+	    memcpy(&in6_pktinfo, CMSG_DATA(cmsghdr), sizeof(struct in6_pktinfo));
+	    if (!if_indextoname(in6_pktinfo.ipi6_ifindex = ((struct in6_pktinfo *)CMSG_DATA(cmsghdr))->ipi6_ifindex, ibuf)) {
+	      fprintf(stderr, "%s: can't convert interface index %d to name\n", in6_pktinfo.ipi6_ifindex);
+	      exit(1);
+	    };
+	    memset(&sin6, 0, sizeof(struct sockaddr_in6));
+#if SALEN
+            sin6.sin6_len = sizeof(struct sockaddr_in6);
+#endif /* SALEN */
+            sin6.sin6_family = AF_INET6;
+	    sin6.sin6_addr = in6_pktinfo.ipi6_addr;
+	    if (nrl_getnameinfo((struct sockaddr *)&sin6, sizeof(struct sockaddr_in6), dbuf, sizeof(dbuf), NULL, 0, nflag)) {
+	      fprintf(stderr, "getnameinfo() failed!\n");
+	      exit(1);
+	    };
+	  };
+    };
+#endif /* INET6 && defined(IPV6_PKTINFO) */
 
     if (datasize) {
       datablock[datasize] = 0;
 
+      strcpy(messagebuf, "Got  ");
       if (patternlen)
-        say("Got  %5d bytes from %s.%s", datasize, hbuf, sbuf);
+	sprintf(messagebuf + strlen(messagebuf), "%5d bytes", datasize);
       else
-        say("Got  '%s' from %s.%s", datablock, hbuf, sbuf);
+	sprintf(messagebuf + strlen(messagebuf), "'%s'", datablock);
+
+      sprintf(messagebuf + strlen(messagebuf), " from %s.%s", hbuf, sbuf);
+
+      if (dbuf[0])
+	sprintf(messagebuf + strlen(messagebuf), " to %s", dbuf);
+
+      if (ibuf[0])
+	sprintf(messagebuf + strlen(messagebuf), " via %s", ibuf);
+
+      say(messagebuf);
       fflush(stdout);
-      if (sendto(s2, datablock, datasize, 0, (struct sockaddr *)addrbuf,
-		 addrlen) < 0) {
-	perror("sendto"); 
+      iovec.iov_len = datasize;
+      if (!msghdr2.msg_controllen)
+	msghdr2.msg_control = NULL;
+      if (sendmsg(s2, &msghdr2, 0) < 0) {
+	perror("sendmsg"); 
 	exit(1);
       }
     }
