@@ -1,5 +1,5 @@
 /*
- * $Header: /home/cmetz/src.CVS/inet6-apps/tcpdump-3.2.1/nfsfh.h,v 1.1.1.1 1996/10/26 05:00:50 cmetz Exp $
+ * $Header: /home/cmetz/src.CVS/inet6-apps/tcpdump-3.3.1a2/nfsfh.h,v 1.1 1997/01/15 18:49:09 cmetz Exp $
  *
  * nfsfh.h - NFS file handle definitions (for portable use)
  *
@@ -24,11 +24,11 @@ typedef struct {
  * our internal representation of that.
  */
 typedef	struct {
-	my_devt	fsid_dev;
+	my_devt	Fsid_dev;		/* XXX avoid name conflict with AIX */
 	u_int32_t fsid_code;
 } my_fsid;
 
 #define	fsid_eq(a,b)	((a.fsid_code == b.fsid_code) &&\
-			 dev_eq(a.fsid_dev, b.fsid_dev))
+			 dev_eq(a.Fsid_dev, b.Fsid_dev))
 
 extern void Parse_fh(caddr_t *, my_fsid *, ino_t *, char **, char **, int);
