@@ -50,9 +50,8 @@ static char sccsid[] = "@(#)telnetd.c	8.4 (Berkeley) 5/30/95";
 #include "pathnames.h"
 
 #ifdef INET6
+#include <netdb.h>
 #include <netinet6/in6.h>
-
-#include "support.h"
 #endif /* INET6 */
 
 #if	defined(_SC_CRAY_SECURE_SYS) && !defined(SCM_SECURITY)
