@@ -1,10 +1,11 @@
 /*
+%%% copyright-cmetz
 This software is Copyright 1996 by Craig Metz, All Rights Reserved.
 The Inner Net License Version 2 applies to this software.
 You should have received a copy of the license with this software. If
 you didn't get a copy, you may request one from <license@inner.net>.
 
-v0.02 - NOT FOR REDISTRIBUTION.
+v0.02.
 */
 
 #include <stdio.h>
